package mygame;

import com.jme3.app.SimpleApplication;
import com.jme3.font.BitmapText;
import com.jme3.input.ChaseCamera;
import com.jme3.input.KeyInput;
import com.jme3.input.MouseInput;
import com.jme3.input.controls.ActionListener;
import com.jme3.input.controls.AnalogListener;
import com.jme3.input.controls.KeyTrigger;
import com.jme3.input.controls.MouseButtonTrigger;
import com.jme3.light.DirectionalLight;             // there are other types of lights like light.AmbiemtLight
import com.jme3.math.ColorRGBA;
import com.jme3.math.Quaternion;
import com.jme3.math.Vector3f;
import com.jme3.renderer.Camera;
import com.jme3.renderer.ViewPort;
import com.jme3.scene.Node;
import com.jme3.scene.Spatial;
import com.jme3.texture.Texture;
import com.jme3.util.SkyFactory;

/**
 * Every JME3 game is an instance of the com.jme3.app.SimpleApplication class.
 * The SimpleApplication class is the simplest example of an application: It
 * manages a 3D scene graph, checks for user input, updates the game state, and
 * automatically draws the scene to the screen. These are the core features of a
 * game engine. You extend this simple application and customize it to create
 * your game.
 *
 * @author G. Amador \& A. Gomes
 */
public class Main extends SimpleApplication {

    /*
     * You start every JME3 game from the main() method, as every standard Java
     * application: 
     * 1 - Instantiate your SimpleApplication-based class. 
     * 2 - The application's start() method to start the game engine.
     *
     * @param args
     */
    public static void main(String[] args) {
        Main app = new Main();          // instantiate the game
        //app.setDisplayFps(false);       // TODO: UNCOMMENT TO HIDE FPS INFO ON APPLICATION RUN
        //app.setDisplayStatView(false);  // TODO: UNCOMMENT TO HIDE STATS VIEW INFO ON APPLICATION RUN
        //app.setShowSettings(false);     // TODO: UNCOMMENT TO HIDE SETTINGS MENU ON STARTUP
        app.start();                    // start the game
    }
    protected Node carNode;
    protected Boolean isRunning = true;

    /**
     * Method that is automatically called once at the beginning when the
     * application starts. Every JME3 game must have this method. In the
     * simpleInitApp() method, you load game objects before the game starts.
     */
    @Override
    public void simpleInitApp() {
        // We speedup the movement of the flyby camera default camera
        flyCam.setMoveSpeed(10);

        /* create a SkyBox of dimension */
        /**
         * BEGIN OF EXERCISE 1 (Step 1): uncomment code as indicated in comments
         * in the following TODO's for Step 1.
         */
        // TODO: (Sub-Step 1.1) UNCOMMENT THE FOLLOWING BLOCK
/*        
        Texture westTex = assetManager.loadTexture("Textures/Skybox/SkyBox2_west.jpg");
        Texture eastTex = assetManager.loadTexture("Textures/Skybox/SkyBox2_east.jpg");
        Texture northTex = assetManager.loadTexture("Textures/Skybox/SkyBox2_north.jpg");
        Texture southTex = assetManager.loadTexture("Textures/Skybox/SkyBox2_south.jpg");
        Texture upTex = assetManager.loadTexture("Textures/Skybox/SkyBox2_up.jpg");
        Texture downTex = assetManager.loadTexture("Textures/Skybox/SkyBox2_down.jpg");

        final Vector3f normalScale = new Vector3f(-1, 1, 1);
        Spatial skySpatial = SkyFactory.createSky(
                assetManager,
                westTex,
                eastTex,
                northTex,
                southTex,
                upTex,
                downTex,
                normalScale);
        rootNode.attachChild(skySpatial);
*/
        // END OF EXERCISE 1 (Step 1). -----------------------------------------

        /* adding models to the scene graph */
        /**
         * BEGIN OF EXERCISE 2 (Step 2): complete adding models code in the
         * following TODO's for Step 2.
         */
        // set up the 1st building model (barrack.obj)
        Spatial building1 = assetManager.loadModel(
                "Models/buildings/barrack.obj");
        //building1.rotate(0.0f, -3.0f, 0.0f);                // DO NOT UNCOMMENT! rotate the model -3 radians in the Y axis
        //building1.setLocalTranslation(-5.0f, 0.1f, 0);      // TODO: (Sub-Step 2.1) UNCOMMENT! translate the model -5 in the X axis and 0.1 in the Y axis 
        //building1.scale(0.5f);                              // DO NOT UNCOMMENT! reduce the model to half of its size
        //rootNode.attachChild(building1);                    // DO NOT UNCOMMENT! add a model directly to the scene graph

        // set up the 2nd building model (nuclear-power-plant.obj) and translate it to (5, 0.1, 0)
        // TODO: (Sub-Step 2.2) ADD YOUR CODE HERE!


        // Create a pivot node at (0,0,0) and attach it to the root node, a user defined center point for one or more Geometries is called a pivot.
        // Geometries can transform (e.g. rotate) around their own center, or around a user defined center point.
        Node pivot = new Node("pivot");
        rootNode.attachChild(pivot);        // put this node in the scene

        // Attach buildings 1 and 2 to the *pivot* node. (And transitively to the root node.) */
        pivot.attachChild(building1);
        //pivot.attachChild(building2);   // TODO: (Sub-Step 2.3) UNCOMMENT AFTER BUILDING 2 SET UP CODE IS COMPLETED!
        pivot.scale(0.5f);              // reduce all pivot childs to half of their size

        // add a car model
        Spatial carGeom = assetManager.loadModel(
                "Models/cars/Car.obj");
        carNode = new Node("carNode");
        carNode.attachChild(carGeom);
        carNode.setLocalTranslation(0.0f, 0.0f, 5.0f);  // translate the model 5 in the Z axis 
        carNode.scale(0.1f);                            // reduce the model to 1/10 of its size
        rootNode.attachChild(carNode);                  // add a model directly to the scene graph.

        // add a fighter model located in the project assets/Models/Fighters/ folder and scale it to 1/2 of its size
        // TODO: (Sub-Step 2.4) ADD YOUR CODE HERE!


        // END OF EXERCISE 2 (Step 2). -----------------------------------------

        /* adding lights to the scene graph */
        /**
         * BEGIN OF EXERCISE 3 (Step 3): complete adding lights code in the
         * following TODO's for Step 3.
         */
        // add a light at (10, 20, 10) pointing to (-X. -Y, -Z)
        DirectionalLight sun1 = new DirectionalLight();
        sun1.setDirection(new Vector3f(-10.0f, -20.0f, -10.0f));    // The direction where the light is pointing (-1, -1, -1) and the light position (10, 20, 10)
        rootNode.addLight(sun1);

        // add a light at (-10, 20, -10) pointing to (-X. -Y, -Z)
        // TODO: (Sub-Step 3.1) ADD YOUR CODE HERE! 


        // END OF EXERCISE 3 (Step 3). -----------------------------------------

        /* adding text to the application display view */
        /**
         * BEGIN OF EXERCISE 4 (Step 4): complete adding text code in the
         * following TODO's for Step 4.
         */
        // Display a 1st line of text with a default font
        guiNode.detachAllChildren();                                    // DO NOT COPY THIS LINE WHEN ADDING 2nd, 3rd, etc LINES OF TEXT OR ALL PREVIOUS TEXT WILL BE DISCARDED!
        guiFont = assetManager.loadFont("Interface/Fonts/Default.fnt");
        BitmapText text1 = new BitmapText(guiFont, false);
        text1.setSize(guiFont.getCharSet().getRenderedSize());
        text1.setText("Hello World ...");
        text1.setColor(ColorRGBA.Blue);                                 // set text color to blue
        text1.setLocalTranslation(300, text1.getLineHeight() + 20, 0);  // set text translation to (300, 20, 0)
        guiNode.attachChild(text1);

        // Display a 2nd line of text saying '... and Tim!' with a default font, color red, bellow the 1st line of text
        // TODO: (Sub-Step 4.1) ADD YOUR CODE HERE!       


        // END OF EXERCISE 4 (Step 4). -----------------------------------------

        /* create two camera views */

        // disable the default 1st-person flyCam (don't forget this!!)
        /**
         * BEGIN OF EXERCISE 5 (Step 5): comment/uncomment code as indicated in
         * comments in order to add an additional camera to the scene graph in
         * the following TODO's for Step 5.
         */
        //flyCam.setEnabled(false); // TODO: (Sub-Step 5.1) UNCOMMENT

        // TODO: (Sub-Step 5.2) UNCOMMENT THE FOLLOWING BLOCK       
/*
         // Enable a chase cam
         ChaseCamera chaseCam = new ChaseCamera(cam, carNode, inputManager);

         // Uncomment this to invert the camera's vertical rotation Axis 
         chaseCam.setInvertVerticalAxis(true);

         // Uncomment this to invert the camera's horizontal rotation Axis
         chaseCam.setInvertHorizontalAxis(true);

         // Uncomment this to enable smooth camera motion
         chaseCam.setSmoothMotion(true);

         // Uncomment this to disable trailing of the camera 
         // WARNING, trailing only works with smooth motion enabled. It is true by default.
         chaseCam.setTrailingEnabled(false);

         // Uncomment this to look 3 world units above the target
         chaseCam.setLookAtOffset(Vector3f.UNIT_Y.mult(3));

         // Uncomment this to enable rotation when the middle mouse button is pressed (like Blender)
         // WARNING : setting this trigger disable the rotation on right and left mouse button click
         chaseCam.setToggleRotationTrigger(
         new MouseButtonTrigger(MouseInput.BUTTON_MIDDLE));

         // Uncomment this to set mutiple triggers to enable rotation of the cam
         // Here spade bar and middle mouse button
         chaseCam.setToggleRotationTrigger(
         new MouseButtonTrigger(MouseInput.BUTTON_MIDDLE),
         new KeyTrigger(KeyInput.KEY_SPACE));

         // Setup second, smaller PiP view
         Camera cam2 = cam.clone();
         cam2.setViewPort(.4f, .6f, 0.8f, 1f);
         cam2.setLocation(new Vector3f(-0.10f, 1.57f, 4.81f));
         cam2.setRotation(new Quaternion(0.00f, 0.99f, -0.04f, 0.02f));
         ViewPort viewPort2 = renderManager.createMainView("PiP", cam2);
         viewPort2.setClearFlags(true, true, true);
         viewPort2.attachScene(rootNode);
*/        
        // END OF EXERCISE 5 (Step 5). -----------------------------------------
        
        initKeys(); // load my custom keybinding
    }

    /* Use the main event loop to trigger repeating actions. */
    @Override
    public void simpleUpdate(float tpf) {
        // make the car rotate:
        //car.rotate(0, 2 * tpf, 0);
    }

    /**
     * Custom Key binding: Map named actions to inputs.
     */
    private void initKeys() {
        // map several inputs to one named action
        /**
         * BEGIN OF EXERCISE 6 (Step 6): complete the keys initialization code
         * in the following TODO's for Step 6.
         */
        inputManager.addMapping("Pause", new KeyTrigger(KeyInput.KEY_P));
        inputManager.addMapping("Rotate", new KeyTrigger(KeyInput.KEY_SPACE),
                new MouseButtonTrigger(MouseInput.BUTTON_LEFT));
        inputManager.addMapping("Left", new KeyTrigger(KeyInput.KEY_LEFT));

        // Right Key
        // TODO: (Sub-Step 6.1) ADD YOUR CODE HERE!


        // Up Key
        // TODO: (Sub-Step 6.2) ADD YOUR CODE HERE!


        // Down Key
        // TODO: (Sub-Step 6.3) ADD YOUR CODE HERE!


        // END OF EXERCISE 6 (Step 6). -----------------------------------------

        // add the names to the action/analog listener, i.e., register trigger mappings.
        inputManager.addListener(actionListener, "Pause"); // register the pause action to the ActionListener, because it is an "on/off" action.
        inputManager.addListener(analogListener, "Left", "Right", "Up", "Down", "Rotate"); // register the movement actions to the AnalogListener, because they are gradual actions.        
    }
    private ActionListener actionListener = new ActionListener() {
        @Override
        public void onAction(String name, boolean keyPressed, float tpf) {
            if (name.equals("Pause") && !keyPressed) {
                isRunning = !isRunning;
            }
        }
    };
    private AnalogListener analogListener = new AnalogListener() {
        @Override
        public void onAnalog(String name, float value, float tpf) {
            if (isRunning) {
                if (name.equals("Rotate")) {
                    carNode.rotate(0, value * speed, 0);
                }
                if (name.equals("Left")) {
                    Vector3f v = carNode.getLocalTranslation();
                    carNode.setLocalTranslation(v.x - value * speed, v.y, v.z);
                }

                /**
                 * BEGIN OF EXERCISE 7 (Step 7): complete the keys behavior code
                 * in the following TODO's for Step 7.
                 */
                // Right Key
                // TODO: (Sub-Step 7.1) ADD YOUR CODE HERE!         
                
                
                // Up Key
                if (name.equals("Up")) {
                    Vector3f v = carNode.getLocalTranslation();
                    carNode.setLocalTranslation(v.x, v.y, v.z + value * speed);
                }                
                
                // Down Key
                // TODO: (Sub-Step 7.2) ADD YOUR CODE HERE!
                
                
                // END OF EXERCISE 7 (Step 7). ---------------------------------
            } else {
                System.out.println("Press P to unpause.");
            }
        }
    };
}
